//////////////////////////////////////////////////////////////////////////////

font comic_font = <comic.pcx>, 34, 38;

//////////////////////////////////////////////////////////////////////////////

var video_mode = 7; // 800x600
var video_depth = 32; // 32 bits

var total_chars = 0;
var correct_chars = 0;
var letter; // 1..25
var falling_speed;
var wave = 1; // will increase during the game leading to bigger and bigger falling speeds
var letter_falls = 0; // will be set to 1 as long as the letter is falling

//////////////////////////////////////////////////////////////////////////////

sound beep_sound = <beep.wav>;
sound error_wav = <error.wav>;
sound key_wav = <key.wav>;

//////////////////////////////////////////////////////////////////////////////

string tutor_wmb = <tutor.wmb>;
string temp_str;
string letter_str;

//////////////////////////////////////////////////////////////////////////////

function create_letters(number, interval);
function falling_letters();
function destroy_letter();
function letter_fades();

//////////////////////////////////////////////////////////////////////////////

text start_txt 
{
	font = comic_font;
	pos_x = 0;
	pos_y = 200;
	string = "   Get ready, captain!\n Press any key to begin!";
	flags = d3d, visible;
}

text score_txt
{
	font = comic_font;
	pos_x = 10;
	pos_y = 10;
	string = "Total:\nCorrect:\nWave:";
	flags = d3d, visible;
}

panel score_pan 
{
	pos_x = 0;
	pos_y = 0;
	digits = 250, 10, 3, comic_font, 1, total_chars; 
	digits = 250, 50, 3, comic_font, 1, correct_chars; 
	digits = 250, 90, 3, comic_font, 1, wave; 
	flags = refresh, d3d, visible;
}

//////////////////////////////////////////////////////////////////////////////

function main()
{
	on_d = null; // disable the debugging panel ("D" key)
	randomize(); // generate different random numbers every time we run the game
	fps_max = 30; // limit the frame rate to 30 fps
	level_load (tutor_wmb);	
	sleep (1);
	camera.x = 0;
	camera.y = -350;
	camera.z = 150;
	camera.pan = 90; // rotate it properly
	camera.ambient = 100; // and make it brighter
	while (key_any == 0) {wait (1);}		
	start_txt.string = " First wave - prepare!";
	sleep (2);
	start_txt.visible = off;
	create_letters(10, 2);		
	while(total_chars < 10) {wait (1);} // wait for the first wave to be finished
	sleep (2); // wait for 2 more seconds
	start_txt.string = " Second wave - prepare!";
	start_txt.visible = on; // make the text visible 
	sleep (3); // for 3 seconds	
	start_txt.visible = off; // make the text invisible 
	wave = 2;
	create_letters (10, 1);
	while(total_chars < 20) {wait (1);} // wait for the second wave to be finished
	sleep (2); // wait for 2 more seconds
	start_txt.string = " Third wave - prepare!";
	start_txt.visible = on; // make the text visible 
	sleep (3); // for 3 seconds	
	start_txt.visible = off; // make the text invisible 
	wave = 3;
	create_letters (10, 0.5);
	while(total_chars < 30) {wait (1);} // wait for the third wave to be finished
	sleep (5); // wait for 5 more seconds
	start_txt.string = "   Impressive result!";
	start_txt.visible = on; // make the text visible 
	sleep (5); // for 5 seconds	
	exit; // shut down the engine
}

function create_letters(number, interval)
{
	while (number > 0)
	{
		letter = 1 + int(random(26)); // generate an integer from 1 to 26
		temp.x = 0; // generate the letter in front of the player and above his head
		temp.y = 350;
		temp.z = 450;
		str_for_asc(temp_str, letter + 64); // convert the number to the ascii char associated to the number (65..90)
		str_cat(temp_str, ".wmb"); // add .wmb to create something like this:    p.wmb
		str_cpy(letter_str, temp_str);
		str_trunc(letter_str, (str_len(letter_str) - 1)); // keep only the first character = the letter
		ent_create(temp_str, temp, falling_letters); // now create the letter
		letter_falls = 1;
		while (letter_falls) {wait (1);}
		str_cpy(temp_str, "");
		number -= 1; // decrease the number of letters 
		sleep(interval); // wait
	}
}

function falling_letters()
{	
	wait (1);
	total_chars += 1;
	my.enable_block = on;
	my.event = destroy_letter;
	falling_speed.x = 0;
	falling_speed.y = 0;
	falling_speed.z = -7;
	falling_speed.z *= wave;
	falling_speed.z *= time;
	while ((my != null) && (my.event != null))
	{
		ent_move (falling_speed, nullvector);
		str_for_key(temp_str, key_lastpressed); // convert the scan code to key 
		if (str_cmpi(letter_str, temp_str) && key_any) // compare the falling letter with the key that was pressed
		{ 
			correct_chars += 1; // increase the number of correctly typed characters
			snd_play(key_wav, 30, 0);
			letter_fades();
			str_cpy(letter_str, ""); // don't increase correct_chars several times for a single letter
			while (key_any) {wait (1);}
		}
		else
		{
			if (key_any)
			{
				destroy_letter();	
			}
		}
		wait (1);
	}
}

function destroy_letter()
{
	my.event = null;
	ent_remove (me);
	letter_falls = 0; // the letter has stopped
	snd_play(error_wav, 50, 0);
}
	
function letter_fades()
{
	my.alpha = 100;	
	my.transparent = on;
	while (my.alpha > 5)
	{
		my.alpha -= 10 * time;
		wait (1);
	}
	ent_remove (me);
	letter_falls = 0; // the letter has disappeared, create a new one
}